@@ -12,18 +12,17 @@
 #' * provided directly as expression, character vector, or string
 #' * read from file
 #'
-#' @details
+#' reprex can also be used for syntax highlighting (with or without rendering);
+#' see below for more.
+#'
+#' @section Details:
 #' The usual "code + commented output" is returned invisibly, put on the
 #' clipboard, and written to file. An HTML preview displays in RStudio's Viewer
 #' pane, if available, or in the default browser, otherwise. Leading `"> "`
 #' prompts, are stripped from the input code. Read more at
-#' <http://reprex.tidyverse.org/>.
+#' <https://reprex.tidyverse.org/>.
 #'
-#' reprex sets specific [knitr options](http://yihui.name/knitr/options/),
-#' which you can supplement or override via the `opts_chunk` and
-#' `opts_knit` arguments or via explicit calls to knitr in your reprex code
-#' (see examples). If all you want to override is the `comment` option, use
-#' the dedicated argument, e.g.`commment = "#;-)"`.
+#' reprex sets specific [knitr options](http://yihui.name/knitr/options/):
 #' * Chunk options default to `collapse = TRUE`, `comment = "#>"`,
 #'   `error = TRUE`. Note that `error = TRUE`, because a common use case is bug
 #'   reporting.
@@ -33,6 +32,20 @@
 #'   the packages httr & xml2 or RCurl & XML, depending on your knitr version.
 #'   When `venue = "r"`, `upload.fun` is set to `identity`, so that figures
 #'   remain local. In that case, you may also want to set `outfile`.
+#' You can supplement or override these options with special comments in your
+#' code (see examples).
+#'
+#' @section Syntax highlighting:
+#' A secondary use case for reprex is to produce syntax highlighted code
+#' snippets, with or without rendering, to paste into presentation software,
+#' such as Keynote or PowerPoint. Use `venue = "rtf"`.
+#'
+#' This feature is experimental and currently rather limited. It probably only
+#' works on macOS and requires the installation of the
+#' [highlight](http://www.andre-simon.de/doku/highlight/en/highlight.php)
+#' command line tool, which can be installed via
+#' [homebrew](http://brewformulas.org/Highlight). This venue is discussed in [an
+#' article](https://reprex.tidyverse.org/articles/articles/rtf.html)
 #'
 #' @param x An expression. If not given, `reprex()` looks for code in
 #'   `input` or on the clipboard, in that order.
@@ -47,15 +60,21 @@
 #'   a location and with basename derived from `input`, if sensible, or in
 #'   current working directory with basename derived from [tempfile()]
 #'   otherwise.
-#' @param venue Character. Must be one of the following:
-#' * "gh" for GitHub, the default
-#' * "so" for Stack Overflow
+#' @param venue Character. Must be one of the following (case insensitive):
+#' * "gh" for [GitHub-Flavored Markdown](https://github.github.com/gfm/), the
+#' default
+#' * "so" for [Stack Overflow Markdown](https://stackoverflow.com/editing-help)
 #' * "ds" for Discourse, e.g.,
 #'   [community.rstudio.com](https://community.rstudio.com). Note: this is
 #'   currently just an alias for "gh"!
-#' * "r" or "R" for a runnable R script, with commented output interleaved
+#' * "r" for a runnable R script, with commented output interleaved
+#' * "rtf" for [Rich Text
+#' Format](https://en.wikipedia.org/wiki/Rich_Text_Format) (not supported for
+#' un-reprexing)
 #' @param advertise Logical. Whether to include a footer that describes when and
-#'   how the reprex was created. Read more about [opt()].
+#'   how the reprex was created. If unspecified, the option `reprex.advertise`
+#'   is consulted and, if that is not defined, default is `TRUE` for venues
+#'   `"gh"`, `"so"`, `"ds"`, and `FALSE` for `"r"` and `"rtf"`.
 #' @param si Logical. Whether to include [devtools::session_info()], if
 #'   available, or [sessionInfo()] at the end of the reprex. When `venue` is
 #'   "gh" or "ds", the session info is wrapped in a collapsible details tag.
@@ -66,13 +85,9 @@
 #'   browser). Read more about [opt()].
 #' @param comment Character. Prefix with which to comment out output, defaults
 #'   to `"#>"`. Read more about [opt()].
-#' @param render Logical. Whether to render the reprex or just create the
-#'   templated `.R` file. Defaults to `TRUE`. Mostly for internal testing
-#'   purposes.
-#' @param opts_chunk,opts_knit Named list. Optional
-#'   [knitr chunk and package options](http://yihui.name/knitr/options/) that
-#'   are forwarded to `knitr::opts_chunk$set()` and `knitr::opts_knit$set()`,
-#'   respectively.
+#' @param render Logical. Whether to call [rmarkdown::render()] on the templated
+#'   reprex, i.e. whether to actually run the code. Defaults to `TRUE`. Exists
+#'   primarily for the sake of internal testing.
 #' @param tidyverse_quiet Logical. Sets the option `tidyverse.quiet`, which
 #'   suppresses (`TRUE`, the default) or includes (`FALSE`) the startup message
 #'   for the tidyverse package. Read more about [opt()].
@@ -114,9 +129,7 @@
 #' ## customize the output comment prefix
 #' reprex(rbinom(3, size = 10, prob = 0.5), comment = "#;-)")
 #'
-#' # override a default chunk option, in general
-#' reprex({(y <- 1:4); median(y)}, opts_chunk = list(collapse = FALSE))
-#' # the above is simply shorthand for this and produces same result
+#' # override a default chunk option
 #' reprex({
 #'   #+ setup, include = FALSE
 #'   knitr::opts_chunk$set(collapse = FALSE)
@@ -163,12 +176,16 @@
 #' # write reprex to file AND keep figure local too, i.e. don't post to imgur
 #' tmp <- file.path(tempdir(), "foofy")
 #' reprex({
+#'   #+ setup, include = FALSE
+#'   knitr::opts_knit$set(upload.fun = identity)
+#'
+#'   #+ actual-reprex-code
 #'   #' Some prose
 #'   ## regular comment
 #'   (x <- 1:4)
 #'   median(x)
 #'   plot(x)
-#'   }, outfile = tmp, opts_knit = list(upload.fun = identity))
+#'   }, outfile = tmp)
 #' list.files(dirname(tmp), pattern = "foofy")
 #'
 #' # clean up
@@ -197,99 +214,90 @@
 #' ## include prompt and don't comment the output
 #' ## use this when you want to make your code hard to execute :)
 #' reprex({
+#'   #+ setup, include = FALSE
+#'   knitr::opts_chunk$set(comment = NA, prompt = TRUE)
+#'
+#'   #+ actual-reprex-code
 #'   x <- 1:4
 #'   y <- 2:5
 #'   x + y
-#' }, opts_chunk = list(comment = NA, prompt = TRUE))
+#' })
 #'
 #' ## leading prompts are stripped from source
 #' reprex(input = c("> x <- 1:3", "> median(x)"))
 #' }
 #'
 #' @import rlang
+#' @import fs
 #' @export
 reprex <- function(x = NULL,
                    input = NULL, outfile = NULL,
-                   venue = c("gh", "so", "ds", "r"),
-                   advertise = opt(TRUE),
-                   si = opt(FALSE),
-                   style = opt(FALSE),
-                   show = opt(TRUE),
-                   comment = opt("#>"),
-                   opts_chunk = NULL,
-                   opts_knit = NULL,
+                   venue = c("gh", "so", "ds", "r", "rtf"),
+
+                   render = TRUE,
+
+                   advertise       = NULL,
+                   si              = opt(FALSE),
+                   style           = opt(FALSE),
+                   show            = opt(TRUE),
+                   comment         = opt("#>"),
                    tidyverse_quiet = opt(TRUE),
-                   std_out_err = opt(FALSE),
-                   render = TRUE) {
+                   std_out_err     = opt(FALSE)) {
+
   venue <- tolower(venue)
   venue <- match.arg(venue)
   venue <- ds_is_gh(venue)
+  venue <- rtf_requires_highlight(venue)
 
-  advertise <- arg_option(advertise)
-  si <- arg_option(si)
-  style <- arg_option(style)
-  show <- arg_option(show)
-  comment <- arg_option(comment)
+  advertise       <- advertise %||%
+    getOption("reprex.advertise") %||% (venue %in% c("gh", "so"))
+  si              <- arg_option(si)
+  style           <- arg_option(style)
+  show            <- arg_option(show)
+  comment         <- arg_option(comment)
   tidyverse_quiet <- arg_option(tidyverse_quiet)
-  std_out_err <- arg_option(std_out_err)
-  opts_chunk <- substitute(opts_chunk)
-  opts_knit <- substitute(opts_knit)
+  std_out_err     <- arg_option(std_out_err)
 
+  if (!is.null(input)) stopifnot(is.character(input))
+  if (!is.null(outfile)) stopifnot(is.character(outfile) || is.na(outfile))
   stopifnot(is_toggle(advertise), is_toggle(si), is_toggle(style))
   stopifnot(is_toggle(show), is_toggle(render))
   stopifnot(is.character(comment))
   stopifnot(is_toggle(tidyverse_quiet), is_toggle(std_out_err))
-  if (!is.null(input)) stopifnot(is.character(input))
-  if (!is.null(outfile)) stopifnot(is.character(outfile) || is.na(outfile))
-
-  the_source <- NULL
-  x_captured <- substitute(x)
-  if (!is.null(x_captured) && !is.null(input)) {
-    message("`input` ignored in favor of expression input in `x`.")
-    input <- NULL
-  }
-  infile <- if (is_path(input)) input else NULL
-  the_source <- stringify_expression(x_captured) %||% ingest_input(input)
 
+  x_expr <- enexpr(x)
+  where <- if (is.null(x_expr)) locate_input(input) else "expr"
+  src <- switch(
+    where,
+    expr      = stringify_expression(x_expr),
+    clipboard = ingest_clipboard(),
+    path      = read_lines(input),
+    input     = escape_newlines(sub("\n$", "", input)),
+    NULL
+  )
+  src <- ensure_not_empty(src)
+  src <- ensure_not_dogfood(src)
+  src <- ensure_no_prompts(src)
   if (style) {
-    if (requireNamespace("styler", quietly = TRUE)) {
-      the_source <- styler::style_text(the_source)
-    } else {
-      message("Install the styler package in order to use `style = TRUE`.")
-    }
-  }
-
-  if (venue == "so") {
-    ## empty line between html comment re: syntax highlighting and reprex code
-    the_source <- c("", the_source)
+    src <- ensure_stylish(src)
   }
 
   outfile_given <- !is.null(outfile)
+  infile <- if (where == "path") input else NULL
   files <- make_filenames(make_filebase(outfile, infile))
+
   r_file <- files[["r_file"]]
-  if (would_clobber(r_file)) {
-    return(invisible())
-  }
+  if (would_clobber(r_file)) { return(invisible()) }
   std_file <- if (std_out_err) files[["std_file"]] else NULL
 
-  the_source <- ensure_not_empty(the_source)
-  the_source <- ensure_not_dogfood(the_source)
-  the_source <- ensure_no_prompts(the_source)
-  the_source <- apply_template(c(
-    fodder[[venue]],
-    si = if (isTRUE(si)) .reprex[["session_info"]],
-    comment = comment,
-    upload_fun = if (venue == "r") "identity" else "knitr::imgur_upload",
-    user_opts_chunk = prep_opts(opts_chunk, which = "chunk"),
-    user_opts_knit = prep_opts(opts_knit, which = "knit"),
-    tidyverse_quiet = as.character(tidyverse_quiet),
-    std_file_stub = if (std_out_err) paste0("#' `", std_file, "`\n#'"),
-    advertisement = advertise,
-    body = paste(the_source, collapse = "\n")
-  ))
-  writeLines(the_source, r_file)
+  data <- list(
+    venue = venue, advertise = advertise, si = si,
+    comment = comment, tidyverse_quiet = tidyverse_quiet, std_file = std_file
+  )
+  src <- apply_template(src, data)
+  writeLines(src, r_file)
   if (outfile_given) {
-    message("Preparing reprex as .R file to render:\n  * ", r_file)
+    message("Preparing reprex as .R file:\n  * ", r_file)
   }
 
   if (!render) {
@@ -297,38 +305,48 @@ reprex <- function(x = NULL,
   }
 
   message("Rendering reprex...")
-  ## when venue = "r", the reprex_file != md_file, so we need both
-  reprex_file <- md_file <- reprex_(r_file, std_file)
+  reprex_render(r_file, std_file)
+  ## 1. when venue = "r" or "rtf", the reprex_file != md_file, so we need both
+  ## 2. use our own "md_file" instead of the normalized, absolutized path
+  ##    returned by rmarkdown::render() and, therefore, reprex_()
+  reprex_file <- md_file <- files[["md_file"]]
 
   if (std_out_err) {
-    ## replace backtick'ed std_file with its (lightly processed) contents
-    inject_file(
-      reprex_file,
-      std_file,
-      pre_process = enfence, tag = "standard output and standard error"
-    )
+    ## replace "std_file" placeholder with its contents
+    inject_file(md_file, std_file, tag = "standard output and standard error")
   }
 
   if (outfile_given) {
-    ## pathstem = the common part of the two paths
-    pathstem <- path_stem(r_file, md_file)
-    message("Writing reprex markdown:\n  * ", sub(pathstem, "", md_file))
+    message("Writing reprex markdown:\n  * ", md_file)
   }
 
-  if (identical(venue, "r")) {
-    reprex_file <- files[["rout_file"]]
+  if (venue %in% c("r", "rtf")) {
+    rout_file <- files[["rout_file"]]
     output_lines <- readLines(md_file, encoding = "UTF-8")
-    output_lines <- convert_md_to_r(output_lines, comment = comment)
-    writeLines(output_lines, reprex_file)
+    output_lines <- convert_md_to_r(
+      output_lines, comment = comment, flavor = "fenced"
+    )
+    writeLines(output_lines, rout_file)
     if (outfile_given) {
-      message("Writing reprex as commented R script:\n  * ", reprex_file)
+      message("Writing reprex as commented R script:\n  * ", rout_file)
     }
+    reprex_file <- rout_file
+  }
+
+  if (venue == "rtf") {
+    rtf_file <- files[["rtf_file"]]
+    reprex_highlight(reprex_file, rtf_file)
+    if (outfile_given) {
+      message("Writing reprex as highlighted RTF:\n  * ", reprex_file)
+    }
+    reprex_file <- rtf_file
   }
 
   if (show) {
+    html_file <- files[["html_file"]]
     rmarkdown::render(
       md_file,
-      output_file = files[["html_file"]],
+      output_file = html_file,
       clean = FALSE,
       quiet = TRUE,
       encoding = "UTF-8",
@@ -336,9 +354,9 @@ reprex <- function(x = NULL,
     )
 
     ## html must live in session temp dir in order to display within RStudio
-    files[["html_file"]] <- force_tempdir(files[["html_file"]])
+    html_file <- force_tempdir(html_file)
     viewer <- getOption("viewer") %||% utils::browseURL
-    viewer(files[["html_file"]])
+    viewer(html_file)
   }
 
   out_lines <- readLines(reprex_file, encoding = "UTF-8")
@@ -363,7 +381,7 @@ reprex <- function(x = NULL,
   invisible(out_lines)
 }
 
-reprex_ <- function(input, std_out_err = NULL) {
+reprex_render <- function(input, std_out_err = NULL) {
   callr::r_safe(
     function(input) {
       options(keep.source = TRUE)
@@ -376,12 +394,44 @@ reprex_ <- function(input, std_out_err = NULL) {
   )
 }
 
-convert_md_to_r <- function(lines, comment = "#>") {
-  line_info <- classify_lines_bt(lines, comment = comment)
-  lines <- ifelse(
-    line_info == "prose" & nzchar(lines),
-    paste("#'", lines),
-    lines
+reprex_highlight <- function(rout_file, reprex_file, arg_string = NULL) {
+  arg_string <- arg_string %||% highlight_args()
+  cmd <- paste0(
+    "highlight ", rout_file,
+    " --out-format=rtf --no-trailing-nl --encoding=UTF-8",
+    arg_string,
+    " > ", reprex_file
+  )
+  res <- system(cmd)
+  if (res > 0) {
+    stop("`highlight` call unsuccessful.", call. = FALSE)
+  }
+  res
+}
+
+rtf_requires_highlight <- function(venue) {
+  if (venue == "rtf" && !highlight_found()) {
+    stop(
+      "`highlight` command line tool doesn't appear to be installed.\n",
+      "Therefore, `venue = \"rtf\"` is not supported.",
+      call. = FALSE
+    )
+  }
+  invisible(venue)
+}
+
+highlight_found <- function() Sys.which("highlight") != ""
+
+highlight_args <- function() {
+  hl_style  <-         getOption("reprex.highlight.hl_style", "darkbone")
+  font      <- shQuote(getOption("reprex.highlight.font", "Courier Regular"))
+  font_size <-         getOption("reprex.highlight.font_size", 50)
+  other     <-         getOption("reprex.highlight.other", "")
+
+  paste0(
+    " --style ",     hl_style,
+    " --font ",      font,
+    " --font-size ", font_size,
+    " ", other
   )
-  lines[line_info != "bt"]
 }
